#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define MAX_PALABRA 32

typedef struct {
	char palabra[MAX_PALABRA];
	int valor;
} Frecuencia;

typedef struct {
	Frecuencia *frecuencias;
	int numFrecuencias;
} ListadoFrecuencias;

int contarPalabras(const char *nombreFichero) {
	FILE *fichero = NULL;
	char token[MAX_PALABRA];
	int numPalabras;

	fichero = (FILE *) fopen(nombreFichero, "r");
	if (fichero == NULL) {
		return -1;
	}

	numPalabras = 0;
	while (!feof(fichero)) {
		fscanf(fichero, "%s", token);
		numPalabras++;
	}

	fclose(fichero);

	return numPalabras;
}

ListadoFrecuencias *contarFrecuencias(const char *nombreFichero) {
	FILE *fichero = NULL;
	ListadoFrecuencias *listado = NULL;
	char token[MAX_PALABRA], palabra[MAX_PALABRA];
	int i, j, maxPalabras, encontrado;

	if (nombreFichero == NULL) {
		return NULL;
	}

	// Contamos el numero maximo de palabras
	maxPalabras = contarPalabras(nombreFichero);
	if (maxPalabras == -1) {
		return NULL;
	}

	// Creamos la estructura donde guardar las frecuencias
	listado = (ListadoFrecuencias *) malloc(sizeof(ListadoFrecuencias));
	if (listado == NULL) {
		return NULL;
	}

	listado->frecuencias = (Frecuencia *) malloc(maxPalabras * sizeof(Frecuencia));
	if (listado->frecuencias == NULL) {
		free(listado);
		return NULL;
	}
	listado->numFrecuencias = 0;

	// Contamos las frecuencias de las palabras del fichero
	fichero = (FILE *) fopen(nombreFichero, "r");
	if (fichero == NULL) {
		free(listado->frecuencias);
		free(listado);
		return NULL;
	}

	while (!feof(fichero)) {
		fscanf(fichero, "%s", token);

		// Quitamos los caracteres que no sean letras o numeros
		for (i = 0, j = 0; i < strlen(token); i++) {
			if (ispunct(token[i]) == 0) {
				palabra[j] = tolower(token[i]);
				j++;
			}
		}
		palabra[j] = '\0';

		encontrado = 0;
		for (i = 0; encontrado == 0 && i < listado->numFrecuencias; i++) {
			if (strcmp(listado->frecuencias[i].palabra, palabra) == 0) {
				listado->frecuencias[i].valor++;
				encontrado = 1;
			}
		}
		if (encontrado == 0) {
			strcpy(listado->frecuencias[listado->numFrecuencias].palabra,
					palabra);
			listado->frecuencias[listado->numFrecuencias].valor = 1;
			listado->numFrecuencias++;
		}
	}

	fclose(fichero);

	return listado;
}

void liberarFrecuencias(ListadoFrecuencias *listado) {
	if (listado != NULL) {
		if (listado->frecuencias != NULL)
			free(listado->frecuencias);
		free(listado);
	}
}

int main() {
	ListadoFrecuencias *listado = NULL;
	int i;
	const char *nombreFichero = "quijote.txt";

	listado = contarFrecuencias(nombreFichero);

	if (listado != NULL) {
		for (i = 0; i < listado->numFrecuencias; i++) {
			printf("%s\t%d\n", listado->frecuencias[i].palabra, listado->frecuencias[i].valor);
		}

		liberarFrecuencias(listado);
	}

	return 1;
}
